using System;
using System.Collections;
using Ringlet.Collections;
using Ringlet.Misc.VCS;

namespace Ringlet.CollTest
{
	/// <summary>
	/// A test app for the Ringlet.Collections assembly.
	/// </summary>
	[VCSAttribute("$Ringlet: cs/misc/RingletCollections/CollTest/CollTest.cs,v 1.5 2004/04/09 13:22:31 roam Exp $")]
	class CollTest
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{

			if (args.Length < 1)
			{
				Usage();
				return;
			}
			switch (args[0])
			{
				case "rotdeque":
					TestRotDeque();
					break;

				case "tvector":
					TestTieredVector();
					break;

				default:
					Usage();
					break;
			}
			Console.ReadLine();
		}

		public static void Usage()
		{

			Console.Error.WriteLine(
@"Usage: CollTest component
	rotdeque	- test the RotDeque class
	tvector		- test the TieredVector class");
		}

		public static void TestRotDeque()
		{
			RotDeque d1, d2;
			bool f;
			IEnumerator en;

			try
			{
				d1 = new RotDeque();
				Console.WriteLine("d1: cap {0}, count {1}, empty {2}, full {3}",
				    d1.Capacity, d1.Count, d1.Empty, d1.Full);
				d2 = new RotDeque(5);
				Console.WriteLine("d2: cap {0}, count {1}, empty {2}, full {3}",
				    d2.Capacity, d2.Count, d2.Empty, d2.Full);
				d2.PushBack(2);
				d2.PushFront("abc");
				Console.WriteLine("d2: cap {0}, count {1}, empty {2}, full {3}",
					d2.Capacity, d2.Count, d2.Empty, d2.Full);
				d2.PushFront(1);
				Console.WriteLine("d2.AtFront: {0}", d2.AtFront);
				Console.WriteLine("d2.AtBack: {0}", d2.AtBack);
				Console.WriteLine("d2.PopFront(): {0}", d2.PopFront());
				d2.PushBack("furrfu");
				d2.PushFront("meow");
				d2.PushBack(3.14);
				Console.WriteLine("d2: cap {0}, count {1}, empty {2}, full {3}",
				    d2.Capacity, d2.Count, d2.Empty, d2.Full);
				Console.WriteLine("About to overflow...");
				f = false;
				try
				{
					d2.PushFront("furrblah");
				}
				catch (InvalidOperationException)
				{
					f = true;
				}
				if (!f)
					Console.Error.WriteLine("Did not get an exception!");

				Console.WriteLine("Let's test the enumerator...");
				en = d2.GetEnumerator();
				while (en.MoveNext())
				{
					Console.WriteLine("- {0}", en.Current);
				}

				Console.WriteLine("PopBack: {0}", d2.PopBack());
				Console.WriteLine("Inserting 'whee!' at pos 3...");
				d2.Insert(3, "whee!");
				Console.WriteLine("Let's see the result...");
				en.Reset();
				while (en.MoveNext())
				{
					Console.WriteLine("- {0}", en.Current);
				}
				Console.WriteLine("PopBack: {0}", d2.PopBack());
				Console.WriteLine("Inserting 'wham!' at pos 1...");
				d2.Insert(1, "wham!");
				Console.WriteLine("Let's see the result...");
				en.Reset();
				while (en.MoveNext())
				{
					Console.WriteLine("- {0}", en.Current);
				}
				
				Console.WriteLine("Attempt to clone it...");
				d1 = d2.Clone() as RotDeque;

				Console.WriteLine("front: {0}", d1.PopFront());
				Console.WriteLine("back: {0}", d1.PopBack());
				Console.WriteLine("front: {0}", d1.PopFront());
				Console.WriteLine("back: {0}", d1.PopBack());
				Console.WriteLine("front: {0}", d1.PopFront());
				Console.WriteLine("d1: cap {0}, count {1}, empty {2}, full {3}",
				    d1.Capacity, d1.Count, d1.Empty, d1.Full);
				Console.WriteLine("About to get an empty one");
				f = false;
				try
				{
					Console.Error.WriteLine("Eeek - got {0}", d1.PopFront());
				}
				catch (InvalidOperationException)
				{
					f = true;
				}
				if (!f)
					Console.Error.WriteLine("Did not catch an exception");

				// Seems like all done!
				Console.WriteLine("All tests passed successfully!");
			}
			catch (Exception e)
			{
				Console.Error.WriteLine("\nException: {0}\n{1}",
				    e.Message, e.StackTrace);
			}
		}

		public static void TVectorGenerateValues(int[] vals)
		{
			Random r;
			int i, len;
			
			r = new System.Random();
			len = vals.Length;
			for (i = 0; i < len; i++)
				vals[i] = r.Next(0, len);
		}

		public static void TVectorAddValues(TieredVector v, int[] vals)
		{
			int i, len, pos;

			len = vals.Length;
			Console.Write("Inserting {0} items: ", len);
			for (i = 0; i < len; i++)
			{
				Console.Write("{0} ", vals[i]);
				pos = v.Add(vals[i]);
				if (vals[i] != (int)v[pos])
					throw (new ApplicationException(
					    String.Format(
					    "Inserting {0}th element {1}: reported position {2}, value at position {3}",
					    i + 1, vals[i], pos, v[pos])));
			}
			Console.WriteLine("ok");
		}

		public static void TVectorClear(TieredVector v, int[] vals)
		{
			int i, len;

			len = vals.Length;
			Console.Write("Removing {0} items: ", len);
			for (i = 0; i < len; i++)
			{
				Console.Write(" {0:3d}", vals[i]);
				v.Remove(vals[i]);
			}
			Console.WriteLine("ok");
		}

		public static void TVectorStats(TieredVector v, string name)
		{
			Console.WriteLine("{0}: count {1}, cap {2}, empty {3}, full {4}",
			    name, v.Count, v.Capacity, v.Empty, v.Full);
		}

		public static void TestTieredVector()
		{
			TieredVector v, v2;
			int[] vals;
			int len, i, pos;
			IEnumerator en;

			try
			{
				Console.WriteLine("Creating a tiered vector with n=1");
				v = new TieredVector(1);
				TVectorStats(v, "v");
				vals = new int[4096];
				len = vals.Length;
				TVectorGenerateValues(vals);
				TVectorAddValues(v, vals);
				TVectorStats(v, "v");
				Console.Write("Let's test the enumerator...");
				en = v.GetEnumerator();
				while (en.MoveNext())
					Console.Write(" {0}", en.Current);
				Console.WriteLine(" ok");

				Console.WriteLine("Let's do some soul-searching...");
				for (i = 0; i < len; i++)
				{
					Console.Write(" {0}", vals[i]);
					pos = v.IndexOf(vals[i]);
					Console.Write("@{0}", pos);
					if ((int)v[pos] != vals[i])
						throw (new ApplicationException(String.Format(
						    "At position {0}: expected {1}, found {2}",
						    pos, vals[i], v[pos])));
					else
						Console.Write(".");
				}
				Console.WriteLine(" ok");

				Console.WriteLine("Attempt to clone it...");
				v2 = v.Clone() as TieredVector;
				TVectorStats(v2, "v2");

				Console.WriteLine("Mmkay, let's try this again, this time removing them.");
				for (i = 0; i < len; i++)
				{
					Console.Write(" {0}", vals[i]);
					pos = v2.IndexOf(vals[i]);
					Console.Write("@{0}", pos);
					if ((int)v2[pos] != vals[i])
						throw (new ApplicationException(
						    String.Format("v2[{0}] = {1}, expected {2}",
						    pos, v2[pos], vals[i])));
					v2.RemoveAt(pos);
					Console.Write(".");
				}
				Console.WriteLine(" ok");
				TVectorStats(v2, "v2");
				
				Console.WriteLine("All TieredVector tests passed!");
			}
			catch (Exception e)
			{
				Console.Error.WriteLine("\nException: {0}\n{1}",
				    e.Message, e.StackTrace);
			}
		}
	}
}
